from io import BytesIO

from django.utils.translation import ugettext as _
from django.conf import settings
from django.utils import formats

from reportlab.pdfbase.ttfonts import TTFont
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus import (
    SimpleDocTemplate, Paragraph, Table, TableStyle, Image, Spacer, LongTable)
from reportlab.platypus.flowables import HRFlowable
from reportlab.pdfbase import pdfmetrics
from reportlab.lib import colors
from reportlab.lib.pagesizes import A4


class PdfDocument(object):
    """ """
    story = []
    custom_styles = {}

    def __init__(self):
        """ """
        self.buffer = BytesIO()
        self.styles = getSampleStyleSheet()
        self.define_fonts()
        self.define_styles()

    def define_fonts(self):
        """ """
        pdfmetrics.registerFont(
            TTFont('RobotoRegular', settings.PROJECT_DIR +
                   '/static/fonts/Roboto-Regular-webfont.ttf'))

    def define_styles(self):
        """
        Define custom evote styles for PDF
        """
        self.custom_styles['title-row-h1'] = ParagraphStyle(
            'title-row-h1',
            parent=self.styles['Heading1'],
            fontSize=13.5,
            textColor=colors.HexColor(0x7f96b8),
            textTransform='uppercase',
            fontName='RobotoRegular'
        )
        self.custom_styles['pure-sec-title'] = ParagraphStyle(
            'pure-sec-title',
            parent=self.styles['Heading3'],
            fontSize=12,
            textColor=colors.HexColor(0x2c323f),
            backColor=colors.HexColor(0xf1f4f8),
            fontName='RobotoRegular',
            borderPadding=(8, 0, 8, 15)
        )
        self.custom_styles['label'] = ParagraphStyle(
            'label',
            fontSize=12,
            textColor=colors.HexColor(0x7f96b8),
            leading=15
        )
        self.custom_styles['body'] = ParagraphStyle(
            'body',
            parent=self.styles['BodyText'],
            fontSize=10,
            textColor=colors.black,
            leading=15
        )

    def space(self, width=10, height=10):
        """ Create vertical space """
        return Spacer(width, height)

    def hr(self, width="100%", thickness=1, line_cap='round',
           color=colors.HexColor(0xe4eaf1), space_before=0, space_after=0):
        """ Create horizontal rule """
        return HRFlowable(
            width=width, thickness=thickness, lineCap=line_cap, color=color,
            spaceBefore=space_before, spaceAfter=space_after)

    def title(self, text):
        """ Create title """
        return Paragraph(text, self.custom_styles["title-row-h1"])

    def body(self, text):
        """ Create text with body style """
        return Paragraph(text, self.custom_styles['body'])

    def pure_sec_title(self, text):
        """ Create pure-sec-title text """
        return Paragraph(text, self.custom_styles["pure-sec-title"])

    def label(self, text):
        """ Create label """
        return Paragraph(text, self.custom_styles["label"])

    def image(self, path, width=58.5, height=58.5):
        """ Create image """
        img = Image(path, width=width, height=height)
        return img

    def add_col_space(self, data, col_widths=None, col_space=10):
        """ Adds empty space between collums """
        for i, line in enumerate(data):
            for j, col in enumerate(line):
                if j != 0 and j % 2 != 0:
                    line.insert(j, '')
                    if i == 0:
                        col_widths.insert(j, col_space)

        return data, col_widths

    def get_formated_date(self, date, body=True, icon=True):
        """ Returns the date in defined format """
        src = settings.PROJECT_DIR + '/static/images/calendar.png'
        calendar_icon = ''
        if icon:
            calendar_icon = "<img src='" + src +\
                "' width='21' height='15' valign='-4'></img>"
        result = calendar_icon + formats.date_format(date, 'DATETIME_FORMAT')

        if body:
            return self.body(result)
        return result

    def get_calendar_icon(self, width=23, height=15):
        """ Returns the flowable with calendar icon """
        src = settings.PROJECT_DIR + '/static/images/calendar.png'
        return self.image(src, width=width, height=height)

    def get_reload_icon(self, width=23, height=15):
        """ Returns the flowable wit reload icon """
        src = settings.PROJECT_DIR + '/static/images/reload.png'
        return self.image(src, width=width, height=height)

    def table(self, data, space_before=17, space_after=17, col_widths=None):
        """ Returns simple table """
        table_style = TableStyle()
        table_style.add('VALIGN', (0, 0), (-1, -1), 'TOP')
        table = Table(data, spaceBefore=space_before, spaceAfter=space_after,
                      colWidths=col_widths)
        table.setStyle(table_style)
        return table

    def votes_table(self, data, space_before=17, space_after=17):
        """ Returns table with styles of votes counting """
        table_style = TableStyle()
        data, widths = self.add_col_space(data, col_widths=[150, 150, 150])
        table_style.add('VALIGN', (0, 0), (-1, -1), 'MIDDLE')
        table_style.add('ALIGN', (0, 0), (-1, -1), 'CENTER')
        table_style.add('BOTTOMPADDING', (0, 0), (-1, 0), 5)
        table_style.add('FONTSIZE', (0, 1), (-1, -1), 15)
        table_style.add('LEADING', (0, 1), (-1, -1), 18)
        table_style.add('COLBACKGROUNDS', (0, 1), (-1, -1),
                        (colors.HexColor(0xeeeeee), None))

        table = Table(data, spaceBefore=space_before, spaceAfter=space_after,
                      colWidths=widths)
        table.setStyle(table_style)
        return table

    def long_table(self, data, header=False, col_widths=None):
        """ Returns long table with width 100% and optimized for huge data """
        table_style = TableStyle()
        start_line = 0
        table_style.add('FONTNAME', (0, 0), (-1, -1), 'RobotoRegular')
        table_style.add('FONTSIZE', (0, 0), (-1, -1), 12)
        if header:
            start_line = 1
            table_style.add(
                'TEXTCOLOR', (0, 0), (-1, 0), colors.HexColor(0xa7b7cf))
            table_style.add(
                'LINEBELOW', (0, 0), (-1, 0), 1, colors.HexColor(0xa7b7cf))
            table_style.add('VALIGN', (0, 0), (-1, 0), 'BOTTOM')
            table_style.add('BOTTOMPADDING', (0, 0), (-1, 0), 11)

        table_style.add('VALIGN', (0, start_line), (-1, -1), 'MIDDLE')
        table_style.add('TOPPADDING', (0, start_line), (-1, -1), 12)
        table_style.add('BOTTOMPADDING', (0, start_line), (-1, -1), 12)
        table_style.add(
            'ROWBACKGROUNDS', (0, start_line), (-1, -1),
            (colors.HexColor(0xf1f4f8), None))

        table = ScalaingTable(data, colWidths=col_widths)
        table.setStyle(table_style)
        return table

    def custom_header_table(
            self, data, space_before=17, space_after=17, col_widths=None):
        """ Returns simple table to use as header """
        table_style = TableStyle()
        table_style.add('FONTNAME', (0, 0), (-1, -1), 'RobotoRegular')
        table_style.add('VALIGN', (0, 0), (-1, -1), 'MIDDLE')
        table_style.add('FONTSIZE', (0, 0), (-1, -1), 13.5)
        table_style.add('BACKGROUND', (0, 0), (-1, -1),
                        colors.HexColor(0x5f6e89))
        table_style.add('TOPPADDING', (0, 0), (-1, -1), 0)
        table_style.add('TEXTCOLOR', (0, 0), (-1, -1), colors.white)

        table = ScalaingTable(
            data, spaceBefore=space_before, spaceAfter=space_after,
            colWidths=col_widths, rowHeights=33.5)
        table.setStyle(table_style)
        return table

    def set_story(self):
        """ """
    def get_title(self, **kwargs):
        return _('PDF Document')

    def render(self, **kwargs):
        """ """
        self.set_story(**kwargs)
        doc = SimpleDocTemplate(
            self.buffer, rightMargin=30, leftMargin=40, topMargin=30,
            bottomMargin=30, pagesize=A4, title=self.get_title(**kwargs))
        doc.build(self.story)
        document = self.buffer.getvalue()
        self.buffer.close()
        return document


class ScalaingTable(LongTable):

    def __init__(self, data, *arg, **kw):
        self._scaleWidthPercent = kw.pop('scaleWidth', 100)
        Table.__init__(self, data, *arg, **kw)

    def wrap(self, avail_width, avail_height):
        Table.wrap(self, avail_width, avail_height)
        if self._width < avail_width and self._scaleWidthPercent:
            scale = float(self._scaleWidthPercent) / 100
            scale_width = scale * avail_width / self._width
            self._colpositions = [p * scale_width for p in self._colpositions]
            self._colWidths = [w * scale_width for w in self._colWidths]
            self._width = avail_width * scale
        return (self._width, self._height)
