from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.utils import timezone
from django.conf import settings
from django.apps import apps

from evotevotings.services.routers import CSrouter
from celery.utils.log import get_task_logger
from celery.schedules import crontab
from evote.celery import app
from django_fsm import can_proceed

from .mailings import ElectionNotificationDateMail

logger = get_task_logger(__name__)


def get_election(pk):
    Election = apps.get_model("evoteelections.Election")  # noqa
    return Election.objects.get(pk=pk)


@app.task(ignore_result=True)
def publish_provisional_roll(election_id):
    """Task to change the election state from open to provisional electoral
    roll"""
    election = get_election(election_id)
    if can_proceed(election.publish_provisional_roll):
        election.publish_provisional_roll()
        election.save()
    return election


@app.task(ignore_result=True)
def publish_definitive_roll(election_id):
    """Task to change the election state from provisional electoral roll to
    definitive electoral roll"""
    election = get_election(election_id)
    if can_proceed(election.publish_definitive_roll):
        election.publish_definitive_roll()
        election.save()
    return election


@app.task(ignore_result=True)
def open_election_voting(election_id):
    """Task to change the election state from definitive electoral roll to
    in to vote"""
    election = get_election(election_id)
    if can_proceed(election.voting):
        election.voting()
        election.save()
    return election


@app.task(ignore_result=True)
def close_election_voting(election_id):
    """Task to change the election state from in to vote to in counting"""
    election = get_election(election_id)
    if can_proceed(election.counting):
        election.counting()
        election.save()
    return election


@app.task(ignore_result=True, bind=True)
def close_election_counting(self, election_id):
    """Task to change the election state from in counting to votes scrutinized.
    http://docs.celeryproject.org/en/latest/userguide/tasks.html#retrying
    """
    conf = getattr(settings, "EVOTE_ELECTIONS")
    stop_status = [conf.get('couting_finish_state'),
                   conf.get('couting_error_state')]
    status = CSrouter().check_results_status(election_id)
    # counting in progress
    if status is not None and status not in stop_status:
        raise self.retry(countdown=10, max_retries=None)
    # counting finished
    elif status == conf.get('couting_finish_state'):
        election = get_election(election_id)
        if can_proceed(election.scrutinized):
            election.scrutinized()
            election.save()
        return election
    # error or counting not initialize yet
    else:
        # TODO: log this
        pass
    return None


@app.task(ignore_result=True)
def close_elections(election_id):
    """Task to close all elections"""
    election = get_election(election_id)
    if can_proceed(election.close):
        election.close()
        election.save()
    return election


@app.on_after_finalize.connect
def setup_periodic_tasks(sender, **kwargs):
    """ """
    conf = getattr(settings, "EVOTE_ELECTIONS")
    # declare send_elections_notifications task
    sender.add_periodic_task(
        crontab(
            hour=conf.get('notification_time')[0],
            minute=conf.get('notification_time')[1]
        ),
        send_elections_notifications.s(),
    )


@app.task
def send_elections_notifications():
    """ """
    conf = getattr(settings, "EVOTE_ELECTIONS")
    sent = 0
    Election = apps.get_model("evoteelections.Election") # NOQA
    qs = Election.objects.filter(state__gte=Election.State.OPEN)
    fields = {
        'proposition_begin_date': {
            'subject': _('Evote: Proposition begin notification'),
            'text': _('begin the proposition period')
        },
        'proposition_end_date': {
            'subject': _('Evote: Proposition end notification'),
            'text': _('end the proposition period')
        },
        'minute_end_date': {
            'subject': _('Evote: Minute end notification'),
            'text': _('end the minute period')
        },
        'voting_begin_date': {
            'subject': _('Evote: Voting begin notification'),
            'text': _('begin the voting period')
        },
        'roll_begin_date': {
            'subject': _('Evote: Roll begin notification'),
            'text': _('begin the roll period')
        },
        'roll_end_date': {
            'subject': _('Evote: Roll end notification'),
            'text': _('end the roll period')
        }
    }
    days = conf.get('notification_days_before')
    for election in qs:
        for key in fields:
            date = getattr(election, key)
            for day in days:
                notify_date = date - timezone.timedelta(days=day)
                if notify_date.date() == timezone.now().date():
                    ElectionNotificationDateMail(
                        subject=fields[key]['subject'],
                        context={
                            'object': election,
                            'days': day,
                            'text': fields[key]['text']
                        })\
                        .send(election.electoral_process)
                    sent += 1
    return sent
