from __future__ import unicode_literals

from django.test import SimpleTestCase

from evotevotings.elections.countings import (Vote, ListVote, VoteCounting,
                                              CountingMethods)


class VoteTest(SimpleTestCase):

    def test_unit_weight(self):
        """ """
        result = Vote(1, 5)
        self.assertEqual(result.total, 5)
        self.assertEqual(result.total_weight, 5)

    def test_weight_not_unit(self):
        result = Vote(2, 5)
        self.assertEqual(result.total, 5)
        self.assertEqual(result.total_weight, 10)


class ListVoteTest(SimpleTestCase):

    def setUp(self):
        """ """
        self.weights = {'cat1': 1, 'cat2': 2}
        self.data = {'cat1': 20, 'cat2': 30}

    def test_init(self):
        """ """
        result = ListVote(self.weights, self.data)
        self.assertEqual(result.total, 50)
        self.assertEqual(result.total_weight, 80)

    def test_init_with_unexistent_weight(self):
        """ """
        self.data['cat'] = 20  # cat is not defined on weight object
        with self.assertRaisesMessage(ValueError, "Weight 'cat' don't exists"):
            ListVote(self.weights, self.data)


class VoteCountingBaseTest(SimpleTestCase):

    def setUp(self):
        """ """
        self.weights = {'cat1': 1, 'cat2': 2}
        self.data = {
            'candidates': {
                'cand1': {'cat1': 20, 'cat2': 30},
                'cand2': {'cat1': 30, 'cat2': 20},
            },
            'blank_votes': {'cat1': 20, 'cat2': 30},
            'total_votes': 150,
            'invalid_votes': 10
        }

    def test_init(self):
        """ """
        counting = VoteCounting(self.weights, self.data)
        self.assertEqual(counting.total, 150)
        self.assertEqual(counting.total_weight, 230)
        self.assertEqual(counting.blank_votes.total, 50)
        self.assertEqual(counting.blank_votes.total_weight, 80)
        self.assertEqual(counting.total_votes, self.data['total_votes'])
        self.assertEqual(counting.invalid_votes, self.data['invalid_votes'])

    def test_data(self):
        """ """
        counting = VoteCounting(self.weights, self.data)
        counting.count(seats=100)
        self.assertEqual(counting.data, [r.__dict__ for r in counting])

    def test_proportional_count(self):
        counting = VoteCounting(self.weights, self.data)
        counting.count()
        self.assertEqual(len(counting), 3)
        self.assertEqual(counting[0].cid, 'cand1')
        self.assertEqual(counting[0].total, 80)
        self.assertEqual(counting[0].perc, 34.78260869565217)
        self.assertEqual(counting[0].seats, 0)

    def test_count_reset_the_list(self):
        counting = VoteCounting(self.weights, self.data)
        counting.count()
        self.assertEqual(len(counting), 3)
        counting.count()
        self.assertEqual(len(counting), 3)

    def test_dhondt_count(self):
        """ """
        counting = VoteCounting(self.weights, self.data)
        counting.count(seats=100, method=CountingMethods.DHONDT)
        self.assertEqual(counting.filter('cand1').seats, 53)
        self.assertEqual(counting.filter('cand2').seats, 47)

    def test_dhondt_count_with_empty_candidates(self):
        """ """
        self.data['candidates'] = {}
        counting = VoteCounting(self.weights, self.data)
        counting.count(seats=100, method=CountingMethods.DHONDT)
        self.assertEqual(len(counting), 1)
