from rest_framework.response import Response
from rest_framework.generics import CreateAPIView
from eVotUM.Cripto import (pkiutils,
                           eccblind)
from . import serializers
from evotevotings.voters.mailings import SendReferenceMail
from evotevotings.voters.models import ElectionVoter
from evotevotings.services.validator import utils
from evotevotings.services.routers import ASrouter
from evotevotings.admin.utils import (get_certificates,
                                      validate_certificate)


class FilterSignatureView(CreateAPIView):
    """
    Standard API view
    """
    permission_classes = []
    serializer_class = serializers.FilterSignatureSerializer

    def get_serializer_class(self):
        """
        Define the Serializer class to be interpreted
        by Django Swagger
        """
        return self.serializer_class

    def post(self, request, *args, **kwargs):
        """
        POST method docs
        Returns either true or false
        """
        serializer = serializers.FilterSignatureSerializer(data=request.data)
        # Top-level validation through the serializer
        serializer.is_valid(raise_exception=True)
        sanitized_data = serializer.validated_data
        response_dict = {"result": "false"}
        # Validate the request data
        try:
            # Get the necessary certificates
            certs = get_certificates()

            # Missing voting system certificate
            if 'votesys' not in certs:
                raise Exception('Missing Voting System Certificate')

            # Missing root certificate
            if 'root' not in certs:
                raise Exception('Missing Root Certificate')

            # Missing Validator Service certificate
            if 'vs' not in certs:
                raise Exception('Missing Validator Service Certificate')

            # Check if Voting System certificate is chained to Root
            valid = validate_certificate(certs['votesys'], certs['root'])

            # Not a chained certificate
            if (valid == -1):
                raise Exception('Unchained Voting System certificate')
            # Expired certificate
            if (valid == -2):
                raise Exception('Expired Voting System certificate')

            # Validate signed election_id object
            signed_obj = str(sanitized_data['election_id'])
            (error_code, election_id) = pkiutils.verifyObjectSignature(
                                            signed_obj,
                                            certs['votesys'])

            if(error_code is not None):
                raise Exception('('+str(error_code) +
                                '): Invalid election+user signature')

            # Validate vote object blind signature
            (error_code, unsigned_vote) = eccblind.verifySignature(
                                            certs['vs_pub'],
                                            sanitized_data['signed_vote'],
                                            sanitized_data['blind_components'],
                                            sanitized_data['pr_components'],
                                            sanitized_data['ciphered_vote'])

            if(error_code is not None):
                raise Exception('('+str(error_code) +
                                '): Invalid vote blind signature')

            # Check if vote is available for this user
            vote_status = utils.check_vote_status(
                                                sanitized_data['blind_hash'],
                                                election_id
                                                )
            if(not vote_status):
                raise Exception('Vote not found for the given blind hash')

            return_data = vote_status

            # Remove the hash from the dictionary and send the
            # remaining fields to the anonymizer service
            blind_hash = sanitized_data.pop('blind_hash')
            # Remove the signed election ID aswell
            sanitized_data.pop('election_id')

            # Send to the Anonymizer Service
            if vote_status:
                as_router = ASrouter()
                return_data = as_router.anonymize_vote(
                    election_id,
                    sanitized_data
                )
                # If the response is True, send the reference email
                if return_data:
                    SendReferenceMail({
                        'reference': sanitized_data['unique_bulletin']
                    }).send([(
                        self.request.user.name,
                        self.request.user.get_email()
                    )])
                # Otherwise revert the vote status
                else:
                    election_voter = ElectionVoter.objects.get(
                        blind_vote_hash=blind_hash,
                        election=election_id
                    )
                    election_voter.vote_status = ElectionVoter.\
                        VoteStatus.PENDING
                    election_voter.save()

            # If the response is not true, revert the vote status
            response_dict['result'] = bool(return_data)
            return Response(response_dict)

        except Exception:
            # TODO: Log
            return Response(response_dict)
